/* ownerstring.c -- view/change the owner string on a Toshiba laptop.
 *
 * Copyright (c) 1998/99  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
 *
 * $Log:$
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

static const char rcsid[]="";

#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<unistd.h>
#ifdef __GLIBC__
#include<sys/perm.h>
#endif
#include<signal.h>
#include<paths.h>
#include<readline.h>

#include"sci.h"

static int keyboard = -1;
static int terminal = 0;
static int fd;
struct termio saved;


#define PID_FILE _PATH_VARRUN "ownerstring.pid"

#define USAGE \
"This utility can be used to register the supervisor password and change the\n\
user password mode (allowed or not allowed to run SETUP).\n\n\
Usage: svpw [-u|h]\n\
    without parameters : Display if the supervisor password is registered or\n\
                         not, and allow to register/delete it.\n\
    -u : Display the current user password mode and allow it to be changed.\n"



/*
 * Exit the program closing any open interface to the BIOS if necessary
 */
void leave(int close)
{
	unlink(PID_FILE);

	exit(close);
	return;
}


/*
 * Catch any signals and exit the program in nicely.
 */
void catch(int x)
{
	fprintf(stderr, "ownerstring: caught signal %d, cleaning up...\n", x);
	unlink(PID_FILE);
	exit(1);

	return;
}


/*
 * Print Incompatiable machine type and exit
 */
void Incompatible()
{
	fprintf(stderr, "ownerstring: Incompatiable machine type.\n");
	leave(1);
}




/*
 * Get a new ownerstring from the user
 */
char *GetPassword(char *password)
{
	int loop,scan;


	return NULL;
}


/*
 * Print any existing ownerstring
 */
void PrintOwnerString(void)
{
	int loop;

	printf("FIXME: print ownerstring\n");

	return;
}


int main(int argc, char *argv[])
{
	SciRegisters reg;
	int pid,help,version;
	char *owner;


	/* get the necessary I/O permissions */

	if (ioperm(0xb2, 1, 1)) {
		fprintf(stderr, "ownerstring: can't get I/O permissions.\n");
		return 1;
	}

	/* this program *must* be run as root */

	if (getuid()) {
		fprintf(stderr, "ownerstring: must be run as root.\n");
		return 1;
	}


	/* process command line arguments */

 	help = 0;
	if ((--argc>0) && ((*++argv)[0]=='-'))
		switch (*++argv[0]) {
			case 'h':
				help = 1;
				break;
			default:
				printf("tpasswd: illegal option %s\n", argv[0]);
				return 1;
		}

	/* if we recieve a signal, exit cleanly */

	signal(SIGHUP, catch);
	signal(SIGINT, catch);
	signal(SIGQUIT, catch);
	signal(SIGILL, catch);
	signal(SIGTRAP, catch);
	signal(SIGABRT, catch);
	signal(SIGIOT, catch);
	signal(SIGFPE, catch);
	signal(SIGKILL, catch);
	signal(SIGSEGV, catch);
	signal(SIGPIPE, catch);
 	signal(SIGTERM, catch);
	signal(SIGCHLD, catch);
	signal(SIGCONT, catch);
	signal(SIGSTOP, catch);
	signal(SIGTSTP, catch);
	signal(SIGTTIN, catch);
	signal(SIGTTOU, catch);

	/* check to see if a copy of ownerstring is already running */

	if (!access(PID_FILE, R_OK)) {
		if ((str = fopen(PID_FILE, "r" ))) {
			fscanf(str, "%d", &pid);
			fclose(str);
			fprintf(stderr, "ownerstring: Already running as "
				"process %d.\n", pid);
			return 1;
		}
	}

	/* create the pid file */

	pid = getpid();
	if ((str = fopen(PID_FILE, "w"))) {
		fprintf(str, "%d\n", pid);
		fclose(str);
	}
	
	/* is this a supported machine? */

	if (SciSupportCheck(&version)==SCI_FAILURE)
		Incompatible();

	/* did the user request help? */

	if (help==1) {
		printf(USAGE);
		leave(0);
	}

	PrintOwnerString();

	owner = readline(NULL);

	printf("STRING:\n%s\n", owner);

	return 0;
}
